<?php

/*
 * Plugin: matchOverlay
 * ~~~~~~~~~~~~~~~~~~~
 * Displays a splash on top of the screen with team scores
 *
 * ----------------------------------------------------------------------------------
 * Author:                Chris92, TheM
 * Version:                v0.1a
 * Date:                2013-03-21
 * Copyright:        Christopher "Chris92" Flgel, Max "TheM" Klaversma
 * System:                XAseco/1.14
 * Game:                Trackmania Forever (TMF)
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies:
 *  - none
 */

Aseco::registerEvent('onSync',                                      'ovrly_onSync');
Aseco::registerEvent('onPlayerConnect',                        'ovrly_onPlayerConnect');
Aseco::registerEvent('onEndRound',                                'ovrly_onEndRound');
Aseco::registerEvent('onNewChallenge',                         'ovrly_onNewChallenge');
Aseco::registerEvent('onPlayerInfoChanged',                'ovrly_onPlayerInfoChanged');
Aseco::registerEvent('onRestartChallenge',                'ovrly_onRestartChallenge');

Aseco::addChatCommand('moverlay', 'Command to modify the matchOverlay overlay');

/*
#/////////////////////////////////////////////////////////////////////#
#                         Initialization
#/////////////////////////////////////////////////////////////////////#
*/

function ovrly_onSync($aseco) {
        // Check for the right XAseco-Version
        $xaseco_min_version = '1.14';
        if(defined('XASECO_VERSION')) {
                if(version_compare(XASECO_VERSION, $xaseco_min_version, '<')) {
                        trigger_error('[plugin.ovrly.php] Not supported XAseco version ('. XASECO_VERSION .')! Please update to min. version '. $xaseco_min_version .'!', E_USER_ERROR);
                }
        } else {
                trigger_error('[plugin.ovrly.php] Can not identify the system, "XASECO_VERSION" is unset! This plugin runs only with XAseco/'. $xaseco_min_version .'+', E_USER_ERROR);
        }

        // Make sure the Server is a TMF one.
        if($aseco->server->getGame() != 'TMF') {
                trigger_error('[plugin.ovrly.php] This plugin supports only TMF, can not start with a "'. $aseco->server->getGame() .'" Dedicated-Server!', E_USER_ERROR);
        }

        global $ovrly_state, $ovrly_score, $ovrly_teams, $ovrly_ver, $ovrly_to, $close, $timeout, $mlid;
        $ovrly_teams = Array("Team Blue", "Team Red");                                          // Team names
        $ovrly_score = Array(0, 0);                                           // Initial overall score
        $ovrly_state = false;                                                      // true = Autostart
        $ovrly_to = 2;                                                                // 0 = Send to all, 1 = Only to players, 2 = Only to spectators
        $ovrly_ver = "0.1a";                                                 // Revision number (not always updated)
        $mlid = "9999999";
        $close = false;
        $timeout = 0;
        $message = '{#server}>> {#highlite}mOverlay plugin '.$ovrly_ver.' loaded';
        $aseco->client->query('ChatSendServerMessage', $aseco->formatColors($message));
}

function ovrly_onPlayerConnect($aseco, $player) {
        global $ovrly_to, $close, $timeout, $ovrly_template;
                if($ovrly_to == 0) {
                        //Send to all players
                        ovrly_show($aseco, $player->login);
                } elseif($ovrly_to == 1) {
                        //send only to players
                        if($aseco->isPlayer($player->login))
                                ovrly_show($aseco, $player->login);
                } elseif($ovrly_to == 2) {
                        //Send only to spectators
                        if($aseco->isSpectator($player)) 
                                ovrly_show($aseco, $player->login);
                }
}

function ovrly_onPlayerInfoChanged($aseco, $changes) {
        global $ovrly_to, $close, $timeout, $ovrly_template, $mlid;
                if($ovrly_to == 2) {
                        if($changes['SpectatorStatus'] == 0) { 
                                ovrly_hide($aseco, $changes['Login']);
                        } elseif($changes['SpectatorStatus'] > 0) {
                                ovrly_show($aseco, $changes['Login']);
                        }
                }
                if($ovrly_to == 1) {
                        if($changes['SpectatorStatus'] > 0) { 
                                ovrly_hide($aseco, $changes['Login']);
                        } elseif($changes['SpectatorStatus'] == 0) {
                                ovrly_show($aseco, $changes['Login']);
                        }
                }
}
function ovrly_onNewChallenge($aseco) {
                ovrly_updateAll($aseco);
}

function chat_moverlay($aseco, $command) {
                global $ovrly_teams, $ovrly_state, $ovrly_score, $ovrly_to;
                $admin = $command['author'];
                $login = $admin->login;
                $nick = $admin->nickname;
                
                $cmd = explode(' ', $command['params']);
                
                if($aseco->isMasterAdmin($admin) || $aseco->isAdmin($admin)) {
                        if($cmd[0] == 'team') {
                                $ovrly_teams[($cmd[1]-1)] = substr(str_replace('team ', '', $command['params']), 1);
                                $msg = 'Team '.$cmd[1].' is now named '.$ovrly_teams[($cmd[1]-1)].'.';
                        } elseif($cmd[0] == 'mscore') {
                                $ovrly_score = array($cmd[1], $cmd[2]);
                                $msg = 'Match score set to '.$cmd[1].' - '.$cmd[2].'.';
                        } elseif($cmd[0] == 'activate' || $cmd[0] == 'enable') {
                                $ovrly_state = true;
                                $msg = 'Plugin matchOverlay has been activated.';
                        } elseif($cmd[0] == 'disable') {
                                $ovrly_state = false;
                                $msg = 'Plugin matchOverlay has been disabled.';
                                foreach($aseco->server->players->player_list as $player)
                                    ovrly_hide($aseco, $player->login);
                        } elseif($cmd[0] == 'to') {
                                if($cmd[1] == 'players') {
                                        $ovrly_to = 1;
                                } elseif($cmd[1] == 'spectators') {
                                        $ovrly_to = 2;
                                } else {
                                        $ovrly_to = 0;
                                }
                                
                                $s = array("everyone connected", "players", "spectators");
                                $msg = 'Displaying matchOverlay to '.$s[$ovrly_to].'.';
                        } elseif($cmd[0] == 'about') {
                                $msg = 'Inspired by the original ESL TV tSplash plugin for FAST Aseco by Sven Stucki, matchOverlay gives you a nice overlay which can be used not only for streams. (by Chris92 & TheM)';
                        }
                        
                        if($ovrly_state && $cmd[0] != 'about' && $cmd[0] != 'disable') {
                                ovrly_updateAll($aseco);
                                
                        }
                        if($msg != '')
                                $aseco->client->query('ChatSendServerMessageToLogin', $msg, $login);
                }
}

function ovrly_updateAll($aseco) {
        global $ovrly_to, $close, $timeout, $ovrly_template;
                if($ovrly_to == 0) {
                        //Send to all players
                        foreach($aseco->server->players->player_list as $player) {
                                ovrly_show($aseco, $player->login);
                        }
                } elseif($ovrly_to == 1) {
                        //send only to players
                        foreach($aseco->server->players->player_list as $player) {
                                if(!$aseco->isSpectator($player)) 
                                        ovrly_show($aseco, $player->login);
                        }
                } elseif($ovrly_to == 2) {
                        //Send only to spectators
                        foreach($aseco->server->players->player_list as $player) {
                                if($aseco->isSpectator($player)) {
                                        ovrly_show($aseco, $player->login);
                                }
                        } 
                }
}

function ovrly_show($aseco, $login) {
        global $ovrly_state, $ovrly_score, $ovrly_teams, $ovrly_to, $close, $timeout, $mlid;
        
        if(!$ovrly_state) return;
        
        $aseco->client->query('GetCurrentRanking', 2, 0);
        $tscore = $aseco->client->getResponse();
         
        $xml = '<manialinks>
        <manialink id='.$mlid.'>
         <quad posn="-42.5 45 0" sizen="85 11.3" image="http://chris92.tm-betmania.com/images/scoreboard.png" />
        <label posn="-6 39.5 1" sizen="19.8 5" style="TextRaceChrono" halign="center" valign="center" textsize="8" textcolor="FFFF" text="'.$tscore[0]['Score'].'"></label>
        <label posn="+6 39.5 1" sizen="19.8 5" style="TextRaceChrono" halign="center" valign="center" textsize="8" textcolor="FFFF" text="'.$tscore[1]['Score'].'"></label>
        <label posn="0 36.3 1" sizen="19.8 5" style="TextTitle3" halign="center" valign="center" textsize="2" textcolor="FFFF" text="'.$ovrly_score[0].' - '.$ovrly_score[1].'"></label>
        <label posn="-36 40.5 1" style="TextRankingsBig" valign="center" halign="left" sizen="39 4" textsize="4" textcolor="FFFF" text="'.$ovrly_teams[0].'"></label>
        <label posn="36 40.5 1" style="TextRankingsBig" valign="center" halign="right" sizen="20.8 3.6" textsize="4" textcolor="FFFF" text="'.$ovrly_teams[1].'"></label>
        </manialink>
        </manialinks>';
        
        $aseco->client->query('SendDisplayManialinkPageToLogin', $login, $xml, ($timeout * 1000), $close);
<!--        $aseco->console('[mOverlay] Showing overlay to {1}!', $login);-->
}

function ovrly_hide($aseco, $login) {
        global $close, $mlid;
        $xml = '<manialink id='.$mlid.'></manialink>';
        $aseco->client->query('SendDisplayManialinkPageToLogin', $login, $xml, (1 * 1000), $close);
<!--        $aseco->console('[mOverlay] Hiding overlay from {1} ', $login);-->
}
?>